package vue;

/**
 * Les classes suivantes sont utilises pour viter de grer la mise en page
 * des lments dans la fentre. La classe GridBagLayout hrite de
 * LayoutManager.
 */
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.LayoutManager;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.text.Document;

import controleur.ControleurVue;

/**
 * La classe Vue est la vue du design MVC.
 * 
 * La fentre prsente une liste, une bote de texte et un bouton. Il est
 * possible d'ajouter des mots dans une liste. Pour ajouter un mot, il suffit
 * d'entrer le mot dans la bote de texte et d'appuyer sur le bouton pour
 * ajouter le texte dans la liste. Lors de l'ajout, la bote de texte se
 * videra.
 * 
 * @author Franois Caron
 *
 */
public class Vue extends JFrame implements Observer {

	/**
	 * Ce numro est gnr par Eclipse, car la classe Vue hrite de JFrame
	 */
	private static final long serialVersionUID = 391679701235987820L;
	
	/**
	 * Le contrleur de la vue.
	 * Il coute pour les actions telles que peser sur un bouton et modifier le texte dans une zone.
	 */
	private ControleurVue controleur;
	
	/**
	 * Les lments dans le Jframe
	 */
	private JButton boutonAjouter;
	private JButton boutonRetirer;
	private JList liste;
	private JTextField boiteTexte;
	
	/**
	 * Constructeur sans paramtre
	 */
	public Vue() {
		/*
		 * Dfinit le titre de la fentre.
		 * La mthode "setTitle(...)" sert galement  dfinir le titre de la fentre.
		 */
		super("Squelette MVC");
		
		/*
		 * Indique que la fermeture de la fentre termine l'excution de l'application.
		 */
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		/*
		 * Instanciation du contrleur.
		 * On passe la classe en tant qu'observateur.
		 */
		controleur = new ControleurVue((Observer)this);
		
		/*
		 * Ceci cre un gestionnaire de mise en page.
		 * La classe GridBagConstraints permet de spcifier l'emplacement et autre...
		 * Pour ajouter des lments  la fentre, il faut passer par le "Content Pane".
		 */
		LayoutManager gestionnaireComposants = new GridBagLayout();
		GridBagConstraints contraintes = new GridBagConstraints();
		getContentPane().setLayout(gestionnaireComposants);
		
		/*
		 * La liste doit tre le premier lment de la fentre et elle doit occuper toute la largeur.
		 */
		contraintes.fill = GridBagConstraints.HORIZONTAL;
		contraintes.gridx = 0;
		contraintes.gridy = 0;
		contraintes.gridwidth = 2;
		liste = new JList();
		/*
		 * On ajouter un ListSelectionListener  la liste pour couter
		 * l'vnement o un lment est slectionn dans la liste. De plus, on
		 * indique explicitement qu'un seul lment peut tre slectionn  la
		 * fois.
		 */
		liste.addListSelectionListener(controleur);
		liste.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		/*
		 * On cre un JScrollPane dans lequel on stocke la liste. De cette
		 * faon, la gestion des curseurs est prise en charge automatiquement.
		 * 
		 * a ne sert  rien de conserveur le JScrollPane ici, car on
		 * s'intresse au contenu de la liste. Le JScrollPane agit comme un
		 * dcorateur pour la liste.
		 */
		JScrollPane curseur = new JScrollPane(liste);
		getContentPane().add(curseur, contraintes);
		
		/*
		 * La zone de texte est le second lment de la fentre.
		 */
		contraintes.fill = GridBagConstraints.HORIZONTAL;
		contraintes.gridx = 0;
		contraintes.gridy = 1;
		contraintes.gridwidth = 2;
		boiteTexte = new JTextField();
		/*
		 * Le contrleur implmente l'interface DocumentListener. On utilise
		 * cet interface pour couter l'ajout et le retrait de caractres dans
		 * la zone de texte.
		 * 
		 * Il faut donc rcuprer le "Document" associ  au JTextField.
		 */
		Document document = boiteTexte.getDocument();
		document.addDocumentListener(controleur);
		getContentPane().add(boiteTexte, contraintes);
		
		/*
		 * Les boutons sont les derniers lments de la fentre.
		 */
		contraintes.fill = GridBagConstraints.HORIZONTAL;
		contraintes.gridx = 0;
		contraintes.gridy = 2;
		contraintes.gridwidth = 1;
		/*
		 * L'attribut weigthx est important, car il permet d'avoir des boutons
		 * de la mme largeur. Sans cet attribut, les boutons ne seront pas de
		 * la mme largeur.
		 */
		contraintes.weightx = 0.5;
		boutonAjouter = new JButton("Ajouter");
		/*
		 * Le contrleur implmente galement l'interface ActionListener. Cet
		 * interface est utilis pour rcuprer le message "buttonPressed" d'un
		 * JButton. Pour bien identifier l'action, la mthode setActionCommand
		 * est utilise.
		 */
		boutonAjouter.addActionListener(controleur);
		boutonAjouter.setActionCommand("AJOUTER");
		getContentPane().add(boutonAjouter, contraintes);
		
		contraintes.fill = GridBagConstraints.HORIZONTAL;
		contraintes.gridx = 1;
		contraintes.gridy = 2;
		contraintes.gridwidth = 1;
		contraintes.weightx = 0.5;
		boutonRetirer = new JButton("Retirer");
		/*
		 * Le contrleur implmente galement l'interface ActionListener. Cet
		 * interface est utilis pour rcuprer le message "buttonPressed" d'un
		 * JButton. Pour bien identifier l'action, la mthode setActionCommand
		 * est utilise.
		 */
		boutonRetirer.addActionListener(controleur);
		boutonRetirer.setActionCommand("RETIRER");
		getContentPane().add(boutonRetirer, contraintes);
		
		/*
		 * Cette mthode rgle automatiquement les dimensions de la fentre en
		 * fonction des composants ajouts  la fentre.
		 */
		pack();
		
		/*
		 * Forcer l'affichage de la fentre.
		 */
		setVisible(true);
	}

	@Override
	public void update(Observable arg0, Object arg1) {
		/*
		 * Une JList doit tre popule  l'aide d'un tableau fixe d'objets.
		 */
		liste.setListData((Object[])arg1);
		
		/*
		 * Lorsque le texte a t ajout de la liste, il faut vider la bote de texte.
		 */
		boiteTexte.setText("");
	}
	
	/**
	 * Point d'entre de l'application.
	 */
	public static void main(String args[]) {
		new Vue();
	}

}
